# coding: utf-8
from educommon.objectpack.ui import TabbedEditWindow, BaseListWindow
from educommon.utils.ui import local_template
from m3_ext.ui.containers import ExtContextMenuItem
from m3_ext.ui.containers.containers import ExtContainer
from m3_ext.ui.containers.containers import ExtToolBar
from m3_ext.ui.containers.trees import ExtTree
from m3_ext.ui.controls.buttons import ExtButton
from m3_ext.ui.misc.store import ExtDataStore
from objectpack.ui import ColumnsConstructor
from objectpack.ui import WindowTab
from objectpack.ui import _create_control_for_field
from objectpack.ui import make_combo_box

from ..models import ReportTemplate


class ListWindow(BaseListWindow):

    def _init_components(self):
        super(ListWindow, self)._init_components()

        self.grid.top_bar.button__build = ExtButton(
            text=u'Собрать отчет',
            icon_cls='database_table',
            handler='buildReport',
            disabled=True,
        )
        self.grid.context_menu_row.menuitem__build = ExtContextMenuItem(
            text=u'Собрать отчет',
            icon_cls='database_table',
            handler='buildReport',
        )

    def _do_layout(self):
        super(ListWindow, self)._do_layout()

        self.grid.top_bar.items.insert(0, self.grid.top_bar.button__build)
        self.grid.context_menu_row.items.append(
            self.grid.context_menu_row.menuitem__build
        )

    def set_params(self, params):
        super(ListWindow, self).set_params(params)

        self.template_globals = local_template('list-window.js')

        self.build_action_url = params['build_action_url']


class AvailableColumnsTree(ExtTree):

    u"""Дерево доступных для использования в отчете колонок."""

    class ToolBar(ExtToolBar):

        def __init__(self, *args, **kwargs):
            super(AvailableColumnsTree.ToolBar, self).__init__(*args, **kwargs)

            self.button__add = ExtButton(
                text=u'Добавить в шаблон',
                icon_cls='add_item',
                handler='addColumnToReport',
                disabled=True,
            )

            self.items[:] = (
                self.button__add,
            )

    def __init__(self, *args, **kwargs):
        kwargs.update(
            title=u'Доступные колонки',
            header=True,
            read_only=True,
        )
        super(AvailableColumnsTree, self).__init__(*args, **kwargs)

        self.style = {'border-width': '1px'}

        ColumnsConstructor.from_config((
            dict(
                data_index='title',
                header=u'Наименование',
                sortable=False,
            ),
        )).configure_grid(self)

        self.top_bar = self.ToolBar()

    def set_params(self, params):
        self.available_columns_url = params['available_columns_action_url']


class ReportColumnsTree(ExtTree):

    u"""Дерево используемых в шаблоне отчета колонок."""

    class ToolBar(ExtToolBar):

        def __init__(self, *args, **kwargs):
            super(ReportColumnsTree.ToolBar, self).__init__(*args, **kwargs)

            self.button__remove = ExtButton(
                text=u'Убрать колонку',
                icon_cls='delete_item',
                handler='removeColumnFromReport',
                disabled=True,
            )

            self.items[:] = (
                self.button__remove,
            )

    def __init__(self, *args, **kwargs):
        kwargs.update(
            title=u'Колонки в отчете',
            header=True,
        )
        super(ReportColumnsTree, self).__init__(*args, **kwargs)

        self.enable_sort = False

        self.style = {'border-width': '1px'}

        ColumnsConstructor.from_config((
            dict(
                data_index='title',
                header=u'Наименование',
                sortable=False,
            ),
        )).configure_grid(self)

        self.top_bar = self.ToolBar()

    def set_params(self, params):
        pass


class ColumnsTab(WindowTab):

    u"""Вкладка "Колонки" окна редактирования шаблона отчета."""

    title = u'Колонки'

    def init_components(self, win):
        super(ColumnsTab, self).init_components(win)

        self.grid__available_columns = AvailableColumnsTree()
        self.grid__report_columns = ReportColumnsTree()
        # ---------------------------------------------------------------------

    def do_layout(self, win, tab):
        super(ColumnsTab, self).do_layout(win, tab)

        win.tab__columns = tab
        tab.grid__available_columns = self.grid__available_columns
        tab.grid__report_columns = self.grid__report_columns

        tab.items[:] = (
            self.grid__available_columns,
            ExtContainer(width=5),
            self.grid__report_columns,
        )

        tab.layout = 'hbox'
        tab.layout_config = {'align': 'stretch'}
        self.grid__available_columns.flex = 2
        self.grid__report_columns.flex = 3

    def set_params(self, win, params):
        super(ColumnsTab, self).set_params(win, params)

        self.grid__available_columns.set_params(params)
        self.grid__report_columns.set_params(params)
        self.grid__report_columns.drag_drop = True
        self.grid__report_columns.handler_beforedrop = 'onBeforeDrop'


class EditWindow(TabbedEditWindow):

    u"""Окно добавления/редактирования шаблона отчета."""

    tabs = (
        ColumnsTab,
    )

    def _init_components(self):
        super(EditWindow, self)._init_components()

        # Нужно для использования в js-шаблоне, т.к. атрибуты с "_" недоступны.
        self.tab_panel = self._tab_container

        self.field__title = _create_control_for_field(
            getattr(ReportTemplate, '_meta').get_field('title'),
            allow_blank=False,
            anchor='100%',
        )
        self.field__data_source_name = make_combo_box(
            name='data_source_name',
            label=u'Источник данных',
            display_field='title',
            value_field='name',
            allow_blank=False,
            anchor='100%',
        )
        self.field__format = _create_control_for_field(
            getattr(ReportTemplate, '_meta').get_field('format'),
            anchor='100%',
        )

        self.panel__top = ExtContainer(
            layout='form',
            label_width=190,
        )

    def _do_layout(self):
        super(EditWindow, self)._do_layout()

        self.panel__top.items[:] = (
            self.field__title,
            self.field__data_source_name,
            self.field__format,
        )

        self.form.layout = 'vbox'
        self.form.layout_config = {'align': 'stretch'}
        self.panel__top.height = 80
        self.panel__top.flex = 0
        self._tab_container.flex = 1

        self.form.items.insert(0, self.panel__top)

    def set_params(self, params):
        super(EditWindow, self).set_params(params)

        self.height, self.width = 600, 1000
        self.template_globals = local_template('edit-window.js')

        self.field__data_source_name.store = ExtDataStore(tuple(
            (name, data_source.title)
            for name, data_source in params['data_sources']
        ))
        self.field__data_source_name.store.id_property = 'name'
        self.field__data_source_name.store.reader.set_fields('title')

        self.columns = params['columns']
